package com.bytesw.bytebot.model.airport;

import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.EqualsAndHashCode;
import lombok.Getter;
import lombok.Setter;
import lombok.ToString;
import org.springframework.format.annotation.DateTimeFormat;

import javax.persistence.*;
import java.io.Serializable;
import java.util.Date;
import java.util.List;

@Entity
@Table(name="itinerary_reservations")
@Getter
@Setter
public class ItinerarioReserva implements Serializable {
    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    @Column(name = "reservation_id", nullable = false)
    private Integer id;

    @Column(name="date_reservation_made")
    @DateTimeFormat(pattern = "yyyy-MM-dd")
    private Date diaReservacion;

    @Column(name="reservation_status_code")
    private String estado;

    @ManyToOne(fetch = FetchType.LAZY, optional = false)
    @JoinColumn(name = "pasajero_id", referencedColumnName = "id")
    @JsonProperty(access = JsonProperty.Access.WRITE_ONLY)
    private Pasajero pasajero;

    @ManyToOne(fetch = FetchType.LAZY, optional = false)
    @JoinColumn(name = "agente_id")
    @JsonProperty(access = JsonProperty.Access.WRITE_ONLY)
    private AgenteReserva agente;

    @ManyToMany(fetch = FetchType.LAZY,
            cascade = {
                    CascadeType.PERSIST,
                    CascadeType.MERGE,
                    CascadeType.ALL
            })
    @JoinTable(name = "itinerary_legs",
            joinColumns = { @JoinColumn(name = "reservation_id") },
            inverseJoinColumns = { @JoinColumn(name = "leg_id") })
    private List<Escala> escalas;

    @ManyToMany(fetch = FetchType.LAZY,
            cascade = {
                    CascadeType.PERSIST,
                    CascadeType.MERGE
            })
    @JoinTable(name = "reservation_payments",
            joinColumns = { @JoinColumn(name = "reservation_id") },
            inverseJoinColumns = { @JoinColumn(name = "payment_id") })
    private List<Pago> pagos;

    @PrePersist
    public void prePersist(){
        diaReservacion = new Date();
    }

    public void addEscala(Escala escala) {
        this.escalas.add(escala);
        escala.getItinerarioReserva().add(this);
    }

    public void addPago(Pago pago) {
        this.pagos.add(pago);
        pago.getItinerarioReserva().add(this);
    }
}

